/* Process model C form file: RsvpSender.pr.c */
/* Portions of this file Copyright (C) MIL 3, Inc. 1992 */



/* OPNET system definitions */
#include <opnet.h>
#include "RsvpSender.pr.h"
FSM_EXT_DECS



/* Header block */
#include<string.h>

#define PK_ARRIVAL op_intrpt_type() == OPC_INTRPT_STRM
#define JOIN_TER  (op_intrpt_type() == OPC_INTRPT_SELF && op_intrpt_code() == 0 )
#define SEND_DATA  (op_intrpt_type() == OPC_INTRPT_SELF && op_intrpt_code() == 1 )
/*  #define SEND_RESERVE  (op_intrpt_type() == OPC_INTRPT_SELF && op_intrpt_code() == 2 )  */
#define END_SIM    op_intrpt_type() == OPC_INTRPT_ENDSIM

#define IGMP_STREAM		1
#define RSVP_STREAM		0
#define IP_STREAM		2

#define InLIST_SIZE		100

#define ON				1
#define OFF				0

#define SESSION_MSG		100
#define SENDER_MSG		101
#define RESERVE_MSG		102
#define RELEASE_MSG		103
#define DATA_MSG		104

#define MULTICAST			500
#define DEFAULT_DEST_PORT	80
#define DEFAULT_SRC_PORT	80
#define DEFAULT_SEND_TTL	10
#define DEFAULT_SEND_BW		60000
#define DEFAULT_SEND_DELAY	1
#define DEFAULT_RECV_BW		50000
#define DEFAULT_RECV_DELAY	1

#define UDP_PROTOCOL_ID 20
#define MAX_INIT_TIME   20


#define VL if(OFF)
#define SVL if(OFF)
#define RADL if(OFF)
 
#define MAX_NO_SESSIONS		100
/*
**  IP Address
*/
typedef struct {
    int NetNum;
    int NodeNum;
}IPAddress;

typedef struct{
	IPAddress Address;
	int	Port;
}SenderInfo;

SenderInfo AllSendersList[InLIST_SIZE];
 
/*
** Data Packet Structure
*/
typedef struct{
    IPAddress SrcAddress;
    char    Data[500];
}SendDataPk;

typedef struct {
	IPAddress SenderAddr;
	IPAddress DestAddr;
	int SenderPort;
	double Bandwidth;
	double Delay;
}sd;

sd SessDirectory[MAX_NO_SESSIONS];

typedef struct {
	char Message[500];
}App_Data;




/* State variable definitions */
typedef struct
	{
	FSM_SYS_STATE
	Objid                   		sv_MyObjId;
	Objid                   		sv_RsvpObjId;
	Objid                   		sv_IpObjId;
	int                     		sv_InStrm;
	int                     		sv_GrpId;
	int                     		sv_Sess;
	int                     		sv_NoOfGrps;
	int                     		sv_CurrSessionId;
	int                     		sv_SessionId;
	int                     		sv_RegSes[InLIST_SIZE];
	double                  		sv_Time;
	double                  		sv_DataRecv;
	double                  		sv_SessMin;
	double                  		sv_SessMax;
	double                  		sv_DataIat;
	double                  		sv_SessIat;
	int                     		sv_IsSender;
	int                     		sv_IsReceiver;
	Distribution*           		sv_GrpDist;
	Distribution*           		sv_TimeDist;
	Distribution*           		sv_TermDist;
	IPAddress               		sv_LocalAddress[InLIST_SIZE];
	} RsvpSender_state;

#define pr_state_ptr            		((RsvpSender_state*) SimI_Mod_State_Ptr)
#define MyObjId                 		pr_state_ptr->sv_MyObjId
#define RsvpObjId               		pr_state_ptr->sv_RsvpObjId
#define IpObjId                 		pr_state_ptr->sv_IpObjId
#define InStrm                  		pr_state_ptr->sv_InStrm
#define GrpId                   		pr_state_ptr->sv_GrpId
#define Sess                    		pr_state_ptr->sv_Sess
#define NoOfGrps                		pr_state_ptr->sv_NoOfGrps
#define CurrSessionId           		pr_state_ptr->sv_CurrSessionId
#define SessionId               		pr_state_ptr->sv_SessionId
#define RegSes                  		pr_state_ptr->sv_RegSes
#define Time                    		pr_state_ptr->sv_Time
#define DataRecv                		pr_state_ptr->sv_DataRecv
#define SessMin                 		pr_state_ptr->sv_SessMin
#define SessMax                 		pr_state_ptr->sv_SessMax
#define DataIat                 		pr_state_ptr->sv_DataIat
#define SessIat                 		pr_state_ptr->sv_SessIat
#define IsSender                		pr_state_ptr->sv_IsSender
#define IsReceiver              		pr_state_ptr->sv_IsReceiver
#define GrpDist                 		pr_state_ptr->sv_GrpDist
#define TimeDist                		pr_state_ptr->sv_TimeDist
#define TermDist                		pr_state_ptr->sv_TermDist
#define LocalAddress            		pr_state_ptr->sv_LocalAddress




/* Process model interrupt handling procedure */



void
RsvpSender ()
	{
	Packet* IgmpApplPkptr;
	Packet* RsvpApplPkptr;
	Packet* DataApplPkptr;
	
	int RecvMsgType;
	int RecvGrpId;
	int RecvSessionId;
	
	int DataAddr1, DataAddr2, DataAddr3;
	FILE* fileptr;


	FSM_ENTER (RsvpSender)

	FSM_BLOCK_SWITCH
		{
		/*---------------------------------------------------------*/
		/** state (Init) enter executives **/
		FSM_STATE_ENTER_FORCED (0, state0_enter_exec, "Init")
			{
			MyObjId = op_id_self();
			RsvpObjId = op_topo_assoc( MyObjId, OPC_TOPO_ASSOC_OUT, OPC_OBJMTYPE_MODULE, 1 );
			IpObjId = op_topo_assoc( RsvpObjId, OPC_TOPO_ASSOC_OUT, OPC_OBJMTYPE_MODULE, 0 );
			
			DataRecv = 0;
			
			op_ima_obj_attr_get( MyObjId, "NGRPS", &NoOfGrps );
			op_ima_obj_attr_get( MyObjId, "SESSMIN", &SessMin );
			op_ima_obj_attr_get( MyObjId, "SESSMAX", &SessMax );
			op_ima_obj_attr_get( MyObjId, "DATA_IAT", &DataIat );
			op_ima_obj_attr_get( MyObjId, "SESS_IAT", &SessIat );
			op_ima_obj_attr_get( MyObjId, "Sender", &IsSender );
			op_ima_obj_attr_get( MyObjId, "Receiver", &IsReceiver );
			
			/*  LocalAddress[0] = ( IPAddress *)op_ima_obj_svar_get( RsvpObjId, "LocalAddress[0]" );  */
			
			op_ima_obj_attr_get( IpObjId, "net_num0", &LocalAddress[0].NetNum );
			op_ima_obj_attr_get( IpObjId, "node_num0", &LocalAddress[0].NodeNum );
			if(OFF)	VL printf("LOCAL ADDRESS : %d.%d. SENDER = %d, RECEIVER = %d\n", 
					   LocalAddress[0].NetNum, LocalAddress[0].NodeNum, IsSender, IsReceiver );
			/**
			*** Generate a Random Variable between 0 and  Max. No of Groups
			*** to select which Grp id to join
			**/
			
			GrpDist = op_dist_load( "uniform_int", 0, NoOfGrps );
			
			/**
			*** Time for the Application to join intially
			**/
			/*  TimeDist = op_dist_load( "exponential", SessIat, 0 );  */
			TimeDist = op_dist_load( "uniform_int", 10, SessIat ); 
			Time = op_dist_outcome( TimeDist );  
			op_intrpt_schedule_self( (op_sim_time() + Time), 0 );
			
			Sess = 0;
			
			/**
			*** Session termiantion distrubution
			**/
			/*  TimeDist = op_dist_load( "exponential", SessIat, 0 );  */
			}


		/** state (Init) exit executives **/
		FSM_STATE_EXIT_FORCED (0, state0_exit_exec, "Init")
			{
			}


		/** state (Init) transition processing **/
		FSM_TRANSIT_FORCE (1, state1_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (Idle) enter executives **/
		FSM_STATE_ENTER_UNFORCED (1, state1_enter_exec, "Idle")
			{
			}


		/** blocking after enter executives of unforced state. **/
		FSM_EXIT (3,RsvpSender)


		/** state (Idle) exit executives **/
		FSM_STATE_EXIT_UNFORCED (1, state1_exit_exec, "Idle")
			{
			}


		/** state (Idle) transition processing **/
		FSM_INIT_COND (PK_ARRIVAL)
		FSM_TEST_COND (END_SIM)
		FSM_TEST_COND (JOIN_TER)
		FSM_TEST_COND (SEND_DATA)
		FSM_TEST_LOGIC ("Idle")

		FSM_TRANSIT_SWITCH
			{
			FSM_CASE_TRANSIT (0, 2, state2_enter_exec, ;)
			FSM_CASE_TRANSIT (1, 5, state5_enter_exec, ;)
			FSM_CASE_TRANSIT (2, 3, state3_enter_exec, ;)
			FSM_CASE_TRANSIT (3, 4, state4_enter_exec, ;)
			}
		/*---------------------------------------------------------*/



		/** state (Arr) enter executives **/
		FSM_STATE_ENTER_FORCED (2, state2_enter_exec, "Arr")
			{
			InStrm = op_intrpt_strm();
			RsvpApplPkptr = op_pk_get( op_intrpt_strm() );
			switch(InStrm)
			{
			    case 0 :
					op_pk_nfd_get( RsvpApplPkptr, "UpcallType", &RecvMsgType );
			        if( RecvMsgType == SESSION_MSG )
			        {
			            op_pk_nfd_get( RsvpApplPkptr, "SessionId", &RecvSessionId );
						VL	SVL printf("Received Session Id is %d\n", RecvSessionId );
			            RegSes[RecvSessionId] = 1;
						CurrSessionId = RecvSessionId;
			        }
			    break;
			    case 1 :
					{
						SendDataPk* RecvDataPk;
						IPAddress DataSndrAdd;
						RecvDataPk = ( SendDataPk* )op_prg_mem_alloc( sizeof(SendDataPk) );
						memset( RecvDataPk, 0, sizeof(SendDataPk) );
						op_pk_nfd_get( RsvpApplPkptr, "Data", &RecvDataPk );
						DataSndrAdd.NetNum = RecvDataPk->SrcAddress.NetNum;
						DataSndrAdd.NodeNum = RecvDataPk->SrcAddress.NodeNum;
						DataRecv++;
						VL  printf("Data Received from %d.%d\n", DataSndrAdd.NetNum, DataSndrAdd.NodeNum );
					}
			    break;
			    default :
			        VL  printf("Packet has arrived from unrecognized stream\n");
			    break;
			 
			}
			 
			op_pk_destroy( RsvpApplPkptr );
			
			}


		/** state (Arr) exit executives **/
		FSM_STATE_EXIT_FORCED (2, state2_exit_exec, "Arr")
			{
			}


		/** state (Arr) transition processing **/
		FSM_TRANSIT_FORCE (1, state1_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (Join) enter executives **/
		FSM_STATE_ENTER_FORCED (3, state3_enter_exec, "Join")
			{
			if(!Sess)
			{
				GrpId = op_dist_outcome( GrpDist );    
			
				/** Make Session call **/
				if(CurrSessionId == 0)
				{
					Packet* SessionPkptr;
					/*  GrpId = 1;  */
					SessionPkptr = op_pk_create_fmt("SessionCall");
					op_pk_nfd_set( SessionPkptr, "SessionId", 0 );
					op_pk_nfd_set( SessionPkptr, "DestNode", GrpId );
					op_pk_nfd_set( SessionPkptr, "DestNet", MULTICAST );
					op_pk_nfd_set( SessionPkptr, "DestPort", DEFAULT_DEST_PORT );
			
					RsvpApplPkptr = op_pk_create_fmt("RSVP_APP_CNTRL");
					op_pk_nfd_set( RsvpApplPkptr, "AppCall", SessionPkptr );
					op_pk_nfd_set( RsvpApplPkptr, "ApplCallType", SESSION_MSG );
			
					op_pk_send( RsvpApplPkptr, RSVP_STREAM );  
				}
			
				/** Make Sender Call **/
				if(IsSender && CurrSessionId > 0)
				{
					Packet* SenderPkptr;
					SenderPkptr = op_pk_create_fmt("SenderCall");
			
					SessDirectory[GrpId].SenderAddr = LocalAddress[0];
					SessDirectory[GrpId].SenderPort = DEFAULT_SRC_PORT;
					SessDirectory[GrpId].DestAddr.NetNum = 500;
					SessDirectory[GrpId].DestAddr.NodeNum = GrpId;
			
					op_pk_nfd_set( SenderPkptr, "SessionId", CurrSessionId );
					op_pk_nfd_set( SenderPkptr, "SourceNet", LocalAddress[0].NetNum );
					op_pk_nfd_set( SenderPkptr, "SourceNode", LocalAddress[0].NodeNum );
					op_pk_nfd_set( SenderPkptr, "SourcePort", DEFAULT_SRC_PORT );
					op_pk_nfd_set( SenderPkptr, "ttl", DEFAULT_SEND_TTL );
					op_pk_nfd_set( SenderPkptr, "BandWidth", DEFAULT_SEND_BW );
					op_pk_nfd_set( SenderPkptr, "Delay", DEFAULT_SEND_DELAY );
			
					SessDirectory[GrpId].Bandwidth = DEFAULT_SEND_BW;
					SessDirectory[GrpId].Delay = DEFAULT_SEND_DELAY;
			
					RsvpApplPkptr = op_pk_create_fmt("RSVP_APP_CNTRL");
					op_pk_nfd_set( RsvpApplPkptr, "AppCall", SenderPkptr );
					op_pk_nfd_set( RsvpApplPkptr, "ApplCallType", SENDER_MSG );
					op_pk_send( RsvpApplPkptr, RSVP_STREAM );
					Sess = 1;
					IgmpApplPkptr = op_pk_create_fmt("MC_APP_CNTRL");
					op_pk_nfd_set( IgmpApplPkptr, "join", 1 );
					op_pk_nfd_set( IgmpApplPkptr, "terminate", 0 );
					op_pk_nfd_set( IgmpApplPkptr, "GRP_ID", GrpId );
					op_pk_send( IgmpApplPkptr, IGMP_STREAM );  
				}
			
				/** Make the IGMP Join Call **/
				if( IsReceiver && CurrSessionId > 0 )
				{
					Packet* ReservePkptr;
					int sender_node, sender_net, sender_port;
					
					if( !(SessDirectory[GrpId].SenderAddr.NodeNum <= 0 || 
						  SessDirectory[GrpId].SenderAddr.NetNum <= 0 || 
						  SessDirectory[GrpId].DestAddr.NodeNum <=0) )
					{
						sender_node = SessDirectory[GrpId].SenderAddr.NodeNum;
						sender_net = SessDirectory[GrpId].SenderAddr.NetNum;
						sender_port = SessDirectory[GrpId].SenderPort;
			
						ReservePkptr = op_pk_create_fmt("ReserveCall");
						op_pk_nfd_set( ReservePkptr, "SessionId", CurrSessionId );
						op_pk_nfd_set( ReservePkptr, "SenderNode", sender_node );
						op_pk_nfd_set( ReservePkptr, "SenderNet", sender_net );
						op_pk_nfd_set( ReservePkptr, "SenderPort", sender_port );
						op_pk_nfd_set( ReservePkptr, "BandWidth", DEFAULT_RECV_BW );
						op_pk_nfd_set( ReservePkptr, "Delay", DEFAULT_RECV_DELAY );
			
						RsvpApplPkptr = op_pk_create_fmt("RSVP_APP_CNTRL");
						op_pk_nfd_set( RsvpApplPkptr, "AppCall", ReservePkptr );
						op_pk_nfd_set( RsvpApplPkptr, "ApplCallType", RESERVE_MSG );
						op_pk_send( RsvpApplPkptr, RSVP_STREAM );
						Sess = 1;
						IgmpApplPkptr = op_pk_create_fmt("MC_APP_CNTRL");
						op_pk_nfd_set( IgmpApplPkptr, "join", 1 );
						op_pk_nfd_set( IgmpApplPkptr, "terminate", 0 );
						op_pk_nfd_set( IgmpApplPkptr, "GRP_ID", GrpId );
						op_pk_send( IgmpApplPkptr, IGMP_STREAM );  
					}
				}
			
				VL	RADL	printf( "Sending the join packet to the IGMP Layer \n" );   
			
				/** Schedule the Interputs **/
				/*  op_intrpt_schedule_self( (op_sim_time() + DataIat), 1);  */
				/*  Time = op_dist_outcome( TermDist );     */
				Time = 300;
				op_intrpt_schedule_self( op_sim_time()+Time, 0);
			}
			else
			{
				IgmpApplPkptr = op_pk_create_fmt( "MC_APP_CNTRL" );
				op_pk_nfd_set( IgmpApplPkptr, "join", 0 );
				op_pk_nfd_set( IgmpApplPkptr, "terminate", 1 );
				op_pk_nfd_set( IgmpApplPkptr, "GRP_ID", GrpId );
				op_pk_send( IgmpApplPkptr, IGMP_STREAM );   
				VL	RADL	printf( "Sending the terminate packet to the IGMP Layer \n" );   
				
				Sess = 0; 
				/** Make Release Call **/
				/*
				RsvpApplPkptr = op_pk_create_fmt("RSVP_APP_CNTRL");
				op_pk_nfd_set( RsvpApplPkptr, "ApplCallType", RELEASE_MSG );
				op_pk_nfd_set( RsvpApplPkptr, "SessionId", &SessionId ); 
				op_pk_send( RsvpApplPkptr, RSVP_STREAM ); 
				VL 	RADL	printf( "Made a release call to the local RSVP Daemon \n" );
				*/
			
				/** Schedule the interupts **/
				/*  Time = op_dist_outcome( TermDist );    */
				Time = 300;
				op_intrpt_schedule_self( op_sim_time()+Time, 0);
			}
			}


		/** state (Join) exit executives **/
		FSM_STATE_EXIT_FORCED (3, state3_exit_exec, "Join")
			{
			}


		/** state (Join) transition processing **/
		FSM_TRANSIT_FORCE (1, state1_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (Data_Send) enter executives **/
		FSM_STATE_ENTER_FORCED (4, state4_enter_exec, "Data_Send")
			{
			/*
			if(IsSender)
			{
			SendDataPk* sd;
			sd->SrcAddress.NetNum = LocalAddress[0].NetNum;
			sd->SrcAddress.NodeNum = LocalAddress[0].NodeNum;
			strcpy( sd->Data, "Test String" );
				if(Sess)
				{
					DataApplPkptr = op_pk_create_fmt("DATA_APP_PK");
					op_pk_nfd_set( RsvpApplPkptr, "Data", sd, op_prg_mem_alloc, op_prg_mem_free, 
								   sizeof(SendDataPk) );
					op_pk_send( RsvpApplPkptr, IGMP_STREAM );  
					VL	RADL	printf( "Send some data to the local IGMP Daemon \n" );
			
					op_intrpt_schedule_self( op_sim_time()+(DataIat), 1);
				}
			}
			*/
			
			/*
			if(OFF)
			{
			if(IsSender)
			{
			    App_Data DataSent;
			    Packet* ip_pkptr;
			    sprintf( DataSent.Message, "This was data sending to the host %d.%d by %d.%d \n",
			             dest_net, dest_node, LocalAddress.NetNum, LocalAddress.NodeNum );
			    pkptr = op_pk_create_fmt("Unicast_Appl");
			 
			    op_pk_nfd_set(pkptr, "UniData", &DataSent, op_prg_mem_alloc,
			                  op_prg_mem_free, sizeof(App_Data) );
			 
			    data_len = op_pk_total_size_get( pkptr )/8;
			    ip_pkptr = op_pk_create_fmt("ip_dgram");
			    op_pk_nfd_set(ip_pkptr, "data", pkptr );
			    op_pk_nfd_set(ip_pkptr, "prot_id", UDP_PROTOCOL_ID );
			    op_pk_bulk_size_set(ip_pkptr, data_len*8 );
			    op_pk_nfd_set(ip_pkptr, "dest_net", 500 );
			    op_pk_nfd_set(ip_pkptr, "dest_node", grp_id );
			 
			    op_pk_nfd_set(ip_pkptr, "src_net", LocalAddress.NetNum );
			    op_pk_nfd_set(ip_pkptr, "src_node", LocalAddress.NodeNum );
			 
			    op_pk_nfd_set(ip_pkptr, "orig_len", data_len );
			    op_pk_nfd_set(ip_pkptr, "frag_len", data_len );
			    op_pk_nfd_set(ip_pkptr, "ttl", 2 );
			    op_pk_nfd_set(ip_pkptr, "frag", 0 );
			 	
				VL
				{        
					printf("Sending data packet from Resv-MCAST Appl at %f\n", op_sim_time() );
			    }
			 
			    data_send++;
			    op_stat_local_write( data_sent, data_send);
			 
			    op_pk_send(ip_pkptr, IP_STREAM);
			}
			}
			
			op_intrpt_schedule_self( op_sim_time()+(DataIat), 1);  
			}
			*/
			}


		/** state (Data_Send) exit executives **/
		FSM_STATE_EXIT_FORCED (4, state4_exit_exec, "Data_Send")
			{
			}


		/** state (Data_Send) transition processing **/
		FSM_TRANSIT_FORCE (1, state1_enter_exec, ;)
		/*---------------------------------------------------------*/



		/** state (EndSim) enter executives **/
		FSM_STATE_ENTER_UNFORCED (5, state5_enter_exec, "EndSim")
			{
			}


		/** blocking after enter executives of unforced state. **/
		FSM_EXIT (11,RsvpSender)


		/** state (EndSim) exit executives **/
		FSM_STATE_EXIT_UNFORCED (5, state5_exit_exec, "EndSim")
			{
			}


		/** state (EndSim) transition processing **/
		FSM_TRANSIT_MISSING ("EndSim")
		/*---------------------------------------------------------*/



		}


	FSM_EXIT (0,RsvpSender)
	}




void
RsvpSender_svar (prs_ptr,var_name,var_p_ptr)
	RsvpSender_state		*prs_ptr;
	char			*var_name, **var_p_ptr;
	{

	FIN (RsvpSender_svar (prs_ptr))

	*var_p_ptr = VOS_NIL;
	if (Vos_String_Equal ("MyObjId" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_MyObjId);
	if (Vos_String_Equal ("RsvpObjId" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_RsvpObjId);
	if (Vos_String_Equal ("IpObjId" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_IpObjId);
	if (Vos_String_Equal ("InStrm" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_InStrm);
	if (Vos_String_Equal ("GrpId" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_GrpId);
	if (Vos_String_Equal ("Sess" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_Sess);
	if (Vos_String_Equal ("NoOfGrps" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_NoOfGrps);
	if (Vos_String_Equal ("CurrSessionId" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_CurrSessionId);
	if (Vos_String_Equal ("SessionId" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_SessionId);
	if (Vos_String_Equal ("RegSes" , var_name))
		*var_p_ptr = (char *) (prs_ptr->sv_RegSes);
	if (Vos_String_Equal ("Time" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_Time);
	if (Vos_String_Equal ("DataRecv" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_DataRecv);
	if (Vos_String_Equal ("SessMin" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_SessMin);
	if (Vos_String_Equal ("SessMax" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_SessMax);
	if (Vos_String_Equal ("DataIat" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_DataIat);
	if (Vos_String_Equal ("SessIat" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_SessIat);
	if (Vos_String_Equal ("IsSender" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_IsSender);
	if (Vos_String_Equal ("IsReceiver" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_IsReceiver);
	if (Vos_String_Equal ("GrpDist" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_GrpDist);
	if (Vos_String_Equal ("TimeDist" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_TimeDist);
	if (Vos_String_Equal ("TermDist" , var_name))
		*var_p_ptr = (char *) (&prs_ptr->sv_TermDist);
	if (Vos_String_Equal ("LocalAddress" , var_name))
		*var_p_ptr = (char *) (prs_ptr->sv_LocalAddress);

	FOUT;
	}




void
RsvpSender_diag ()
	{
	Packet* IgmpApplPkptr;
	Packet* RsvpApplPkptr;
	Packet* DataApplPkptr;
	
	int RecvMsgType;
	int RecvGrpId;
	int RecvSessionId;
	
	int DataAddr1, DataAddr2, DataAddr3;
	FILE* fileptr;

	FIN (RsvpSender_diag ())


	FOUT;
	}




void
RsvpSender_terminate ()
	{
	Packet* IgmpApplPkptr;
	Packet* RsvpApplPkptr;
	Packet* DataApplPkptr;
	
	int RecvMsgType;
	int RecvGrpId;
	int RecvSessionId;
	
	int DataAddr1, DataAddr2, DataAddr3;
	FILE* fileptr;

	FIN (RsvpSender_terminate ())


	FOUT;
	}



Compcode
RsvpSender_init (pr_state_pptr)
	RsvpSender_state			**pr_state_pptr;
	{
	static VosT_Cm_Obtype	obtype = OPC_NIL;

	FIN (RsvpSender_init (pr_state_pptr))

	if (obtype == OPC_NIL)
		{
		if (Vos_Catmem_Register ("proc state vars (RsvpSender)",
			sizeof (RsvpSender_state), Vos_Nop, &obtype) == VOSC_FAILURE)
			FRET (OPC_COMPCODE_FAILURE)
		}

	if ((*pr_state_pptr = (RsvpSender_state*) Vos_Catmem_Alloc (obtype, 1)) == OPC_NIL)
		FRET (OPC_COMPCODE_FAILURE)
	else
		{
		(*pr_state_pptr)->current_block = 0;
		FRET (OPC_COMPCODE_SUCCESS)
		}
	}



